"""
basic executing database functions
"""
import sqlite3

from sohstationviewer.conf.dbSettings import dbConf


def executeDB(sql):
    """
    Execute or fetch data from DB
    :param sql: str - request string to execute or fetch data from database
    :return rows: [(str/int,] - result of querying DB
    """
    conn = sqlite3.connect(dbConf['dbpath'])
    cur = conn.cursor()
    try:
        cur.execute(sql)
    except sqlite3.OperationalError as e:
        print("sqlite3.OperationalError:%s\n\tSQL%s" % (str(e), sql))
    rows = cur.fetchall()
    conn.commit()       # used for execute: update/insert/delete
    cur.close()
    conn.close()
    return rows


def executeDB_dict(sql):
    """
    Fetch data and return rows in dictionary with fields as keys
    :param sql: str - request string to fetch data from database
    :return: [dict,] - result of fetching DB in which each row is a dict
    """
    conn = sqlite3.connect(dbConf['dbpath'])
    conn.row_factory = sqlite3.Row
    cur = conn.cursor()
    try:
        cur.execute(sql)
    except sqlite3.OperationalError as e:
        print("sqlite3.OperationalError:%s\n\tSQL%s" % (str(e), sql))
    rows = cur.fetchall()
    cur.close()
    conn.close()
    return [dict(row) for row in rows]


def trunc_addDB(table, sqls):
    """
    Truncate table and refill with new data
    :param table: str - name of data table to process
    :param sqls: [str, ] - list of INSERT query to add values to table
    :return: str: error message if not successful
        or bool(True): if successful
    """
    try:
        conn = sqlite3.connect(dbConf['dbpath'])
        cur = conn.cursor()
        cur.execute('BEGIN')
        cur.execute(f'DELETE FROM {table}')
        for sql in sqls:
            cur.execute(sql)
        cur.execute('COMMIT')
    except sqlite3.Error as e:
        try:
            cur.execute('ROLLBACK')
        except Exception:
            pass
        return (f'Cannot write to table {table}'
                f'because of database error:\n{str(e)}')
    except Exception as e:
        try:
            cur.execute('ROLLBACK')
        except Exception:
            pass
        return (f'Cannot write to table {table} '
                f'because of code error:\n{str(e)}')
    try:
        cur.close()
        conn.close()
    except Exception:
        pass
    return True
